@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # Part of the psychopy_ext library
-# Copyright 2010-2013 Jonas Kubilius
+# Copyright 2010-2014 Jonas Kubilius
 # The program is distributed under the terms of the GNU General Public License,
 # either version 3 of the License, or (at your option) any later version.
 
@@ -18,13 +18,13 @@ import pandas
 import plot
 
 try:
-    import OrderedDict
+    from collections import OrderedDict
 except:
     from exp import OrderedDict
 
 
 def aggregate(df, rows=None, cols=None, values=None,
-    subplots=None, yerr=None, aggfunc='mean', unstack=False,
+    subplots=None, yerr=None, aggfunc='mean', unstacked=False,
     order='natural', add_names=True):
     """
     Aggregates data over specified columns.
@@ -46,7 +46,7 @@ def aggregate(df, rows=None, cols=None, values=None,
         - aggfunc (str or a 1d function)
             A function to use for aggregation. If a string, it is interpreted
             as a `numpy` function.
-        - unstack (bool, default: False)
+        - unstacked (bool, default: False)
             If True, returns an unstacked version of aggregated data (i.e.,
             rows in rows and columns in columns)). Useful for printing and
             other non-plotting tasks.
@@ -59,11 +59,11 @@ def aggregate(df, rows=None, cols=None, values=None,
         A `pandas.DataFrame` where data has been aggregated in the following
         MultiIndex format:
         - columns:
-        
+
           - level 0: subplots
           - level 1 to n-2: rows
           - level n-1: column
-          
+
         - rows:
             yerr
 
@@ -83,7 +83,9 @@ def aggregate(df, rows=None, cols=None, values=None,
     if values is None:
         raise Exception('You must provide the name(s) of the column(s) that '
                         'is/are aggregated.')
-    allconds = [subplots] + rows + cols + yerr
+    if isinstance(subplots, str) or subplots is None:
+        subplots = [subplots]
+    allconds = subplots + rows + cols + yerr
     allconds = [c for c in allconds if c is not None]
 
     if isinstance(aggfunc, str):
@@ -91,11 +93,12 @@ def aggregate(df, rows=None, cols=None, values=None,
             aggfunc = getattr(np, aggfunc)
         except:
             raise
+
     agg = df.groupby(allconds)[values].aggregate(aggfunc)
 
-    groups = [('subplots', [subplots]), ('rows', rows), ('cols', cols),
+    groups = [('subplots', subplots), ('rows', rows), ('cols', cols),
               ('yerr', yerr)]
-    index_names = agg.index.names
+    index_names = [i for i in agg.index.names]  # since it's FrozenList
     g = 0
     for group in groups:
         for item in group[1]:
@@ -105,26 +108,28 @@ def aggregate(df, rows=None, cols=None, values=None,
     agg.index.names = index_names
 
     if yerr[0] is not None:  # if yerr present, yerr is in rows, the rest in cols
+        if isinstance(values, list):
+            agg = agg[values[0]]
         for yr in yerr:
-            agg = agg.unstack(level='yerr.'+yr)
+            agg = unstack(agg, level='yerr.'+yr)
         # seems like a pandas bug here for not naming levels properly
         agg.columns.names = ['yerr.'+yr for yr in yerr]
         agg = agg.T
     else:
         agg = pandas.DataFrame(agg).T
-        
+
     # sort data nicely
     if isinstance(order, dict):
         order_full = order
     else:
         order_full = dict([(col, order) for col in agg.columns.names])
-        
+
     #if order != 'sorted':
     #names = agg.columns.names  # store it; will need it later
     for level, level_ord in order_full.items():
         if isinstance(level_ord, str):
             col = '.'.join(level.split('.')[1:])
-            if level_ord == 'natural':                
+            if level_ord == 'natural':
                 thisord = df[col].unique()  # preserves order
             elif level_ord == 'sorted':
                 thisord = np.unique(df[col])  # doesn't preserve order
@@ -133,13 +138,14 @@ def aggregate(df, rows=None, cols=None, values=None,
         else:
             thisord = level_ord
         order_full[level] = thisord
+
     agg = reorder(agg, order=order_full)
-    #agg.columns.names = names  # buggy pandas        
-        
+    #agg.columns.names = names  # buggy pandas
+
     # rows should become rows, and cols should be cols if so desired
-    if yerr[0] is None and unstack:
+    if yerr[0] is None and unstacked:
         agg = plot._stack_levels(agg, 'rows.')
-            
+
     if not add_names:
         names = []
         for name in agg.columns.names:
@@ -148,7 +154,10 @@ def aggregate(df, rows=None, cols=None, values=None,
                 names.append('.'.join(spl[1:]))
         agg.columns.names = names
 
-    agg.names = values
+    if isinstance(values, list):
+        agg.names = values[0]
+    else:
+        agg.names = values
     return agg
 
 def accuracy(df, values=None, correct='correct', incorrect='incorrect', **kwargs):
@@ -181,6 +190,8 @@ def accuracy(df, values=None, correct='correct', incorrect='incorrect', **kwargs
         correct = [correct]
     if isinstance(incorrect, str):
         incorrect = [incorrect]
+    if isinstance(values, list):
+        values = values[0]
     corr = df[df[values].isin(correct)]
     if len(corr) == 0:
         raise Exception('There are no %s responses' % correct[0])
@@ -191,21 +202,92 @@ def accuracy(df, values=None, correct='correct', incorrect='incorrect', **kwargs
     agg.names = values
     return agg
 
-def confidence(agg, kind='sem', nsamples=None, skipna=True):
-    if isinstance(agg, pandas.DataFrame):
-        mean = agg.mean(skipna=skipna)  # mean across items
-        if kind == 'sem':
-            p_yerr = agg.std(skipna=skipna) / np.sqrt(len(agg))  # std already has ddof=1
+def confidence(agg, kind='sem', within=None, alpha=.05, nsamples=None,
+               skipna=True):
+    """
+    Compute confidence of measurements, such as standard error of means (SEM)
+    or confidence intervals (CI).
 
-        # compute binomial distribution error bars if there is a single sample
-        # only (presumably the number of correct responses, i.e., successes)
-        # from http://en.wikipedia.org/wiki/Binomial_proportion_confidence_interval#Normal_approximation_interval
+    :Args:
+        agg
+    :Kwargs:
+        - kind ('sem', 'ci', or 'binomial', default: 'sem')
+            .. warning:: Binomial not tested throroughly
+        - within (str or list, default: None)
+            For repeated measures designs, error bars are too large.
+            Specify which dimensions come from repeated measures
+            (rows, cols, and/or subplots). It computes
+            within-subject confidence intervals using a method by
+            Loftus & Masson (1994) simplified by Cousinaueu (2005)
+            with Morey's (2008) correction.
+            Based on `Denis A. Engemann's gist <https://gist.github.com/dengemann/6030453>`_
+        - alpha (float, default: .05)
+            For CI and binomial. Computed for single-tail, so effectively
+            alpha/2.
+        - nsamples
+            For binomial distribution confidence intervals if there is
+            a single sample only (which presumably relfects the number
+            of correct responses, i.e., successes). See `Wikipedia
+            <http://en.wikipedia.org/wiki/Binomial_proportion_confidence_interval#Normal_approximation_interval>`_
+        - skipna (default: True)
+            Whether to skip NA / null values or not.
+
+            .. warning:: Not tested thoroughly
+    :Returns:
+        mean, p_yerr
+    """
+    if isinstance(agg, pandas.DataFrame):
+        mean = agg.mean(skipna=skipna)  # mean across participants
+
+        if within is not None:
+            if not isinstance(within, list):
+                within = [within]
+
+            levels = []
+            for dim in within:
+                tmp = [r for r in agg.columns.names if not r.startswith(dim + '.')]
+                levels += tmp
+            if len(levels) > 0:
+                #raise Exception('Could not find levels that start with any of the following: %s' % within)
+            # center data
+            #try:
+                try:
+                    subj_mean = agg.mean(level=levels, skipna=skipna, axis=1)  # mean per subject
+                except:
+                    subj_mean = agg.mean(skipna=skipna)
+            else:
+                subj_mean = agg.mean(skipna=skipna)
+            grand_mean = mean.mean(skipna=skipna)
+            center = subj_mean - grand_mean
+
+            aggc = pandas.DataFrame(index=agg.index, columns=agg.columns)
+            for colname, value in center.iteritems():
+                try:
+                    aggc[colname]  # what a cool bug -- crashes without this line!
+                except:
+                    import pdb; pdb.set_trace()
+                aggc[colname] = (agg[colname].T - value.T).T
+            ddof = 0  # Morey's correction
+        else:
+            aggc = agg.copy()
+            ddof = 1
+
+        if skipna:
+            count = agg.count()  # skips NA by default
+        else:
+            count = len(agg)
+
+        confint = 1 - alpha/2.
+        if kind == 'sem':
+            p_yerr = aggc.std(skipna=skipna, ddof=ddof) / np.sqrt(count)
+        elif kind == 'ci':
+            p_yerr = aggc.std(skipna=skipna, ddof=ddof) / np.sqrt(count)
+            p_yerr *= scipy.stats.t.ppf(confint, count-1)
         elif kind == 'binomial':
-            alpha = .05
-            z = scipy.stats.norm.ppf(1-alpha/2.)
+            z = scipy.stats.norm.ppf(confint)
             p_yerr = z * np.sqrt(mean * (1-mean) / nsamples)
-    else:
-        #mean, p_yerr = self.errorbars(agg, kind='binomial')
+
+    else:  # if there's only one condition? Not sure #FIXME
         mean = agg
         p_yerr = np.zeros((len(agg), 1))
 
@@ -232,7 +314,7 @@ def reorder(agg, order, level=None, dim='columns'):
         - agg (pandas.DataFrame)
             Your (usually aggregated) data
         - order (list or dict)
-            Order of entries. A list is only accepted is `level` is 
+            Order of entries. A list is only accepted is `level` is
             given. Otherwise, order should be a dict with level names
             (str) as keys and a list of order in values.
     :Kwargs:
@@ -242,35 +324,39 @@ def reorder(agg, order, level=None, dim='columns'):
             Whether to reorder rows (or index) or columns.
     :Returns:
         Reordered pandas.DataFrame
-    """     
+    """
     if dim in ['index', 'rows']:
         orig_idx = agg.index
     else:
-        orig_idx = agg.columns           
-        
+        orig_idx = agg.columns
+
+    #if not isinstance(order, dict):
+        ##if level is None:
+            ##raise Exception('When "order" is a list, a "level" must be'
+                            ##'passed as well.')
+        ##else:
+        #try:
+            #orderframe = pandas.DataFrame(orig_idx.tolist())
+            #levelno = orig_idx.names.index(level)
+            #order = {level: orderframe[levelno].unique()}
+        #except: # no levels
+            #order = {level: orig_idx.unique()}
+
     if not isinstance(order, dict):
-        #if level is None:
-            #raise Exception('When "order" is a list, a "level" must be'
-                            #'passed as well.')
-        #else:
-        try:
-            orderframe = pandas.DataFrame(orig_idx.tolist())
-            levelno = orig_idx.names.index(level)
-            order = {level: orderframe[levelno].unique()}
-        except: # no levels
-            order = {level: orig_idx.unique()}
-            
+        order = {level: order}
+
+    #import pdb; pdb.set_trace()
     for levelno, levelname in enumerate(orig_idx.names):
         if not levelname in order:
-            # retain the existing order            
+            # retain the existing order
             this_order = pandas.DataFrame(orig_idx.tolist()) #[lev[levelno] for lev in orig_idx]
-            this_order = this_order[levelno].unique()  # preserves order
+            this_order = this_order.iloc[:, levelno].unique()  # preserves order
             #this_order = np.unique(this_order)
         else:
             this_order = order[levelname]
         # convert to a list
         order[levelname] = [i for i in this_order]
-        
+
     data = np.zeros((len(orig_idx), len(orig_idx.names)))
     ind_sort = pandas.DataFrame(index=orig_idx, data=data, columns=orig_idx.names)
     for rowno, (rowidx, row) in enumerate(ind_sort.iterrows()):
@@ -290,6 +376,46 @@ def reorder(agg, order, level=None, dim='columns'):
         agg_new = agg.reindex(columns=multidx)
     return agg_new
 
+
+def unstack(data, level=-1):
+    """
+    Unstacking that preserves label order, unlike in pandas.
+    """
+    if not isinstance(level, (list, tuple)):
+        levels = [level]
+    else:
+        levels = level
+
+    unstacked = data.unstack(level=level)
+
+    # reorder the newly unstacked levels
+    for level in levels:
+        order = data.index.get_level_values(level).unique()
+        unstacked = reorder(unstacked, order=order, level=level, dim='columns')
+
+    #for level in levels:
+        #order = data.columns.get_level_values(lev).unique()
+        #unstacked = stats.reorder(unstacked, order=order, level=lev, dim='columns')
+
+    return unstacked
+
+def stack(data, level=-1):
+    """
+    Stacking that preserves label order, unlike in pandas.
+    """
+    if not isinstance(level, (list, tuple)):
+        levels = [level]
+    else:
+        levels = level
+
+    stacked = data.stack(level=level)
+
+    for level in levels:
+        order = data.columns.get_level_values(level).unique()
+        stacked = reorder(stacked, order=order, level=level, dim='index')
+
+    return stacked
+
 def df_fromdict(data, repeat=1):
     """
     Produces a factorial DataFrame from a dict or list of tuples.
@@ -403,7 +529,7 @@ def _aggregate_panel(df, rows=None, cols=None, values=None,
             elif func == 'median':
                 agg = df.groupby(allconds)[values].median()
 
-        agg = agg.unstack(yerr)
+        agg = unstack(agg, level=yerr) #agg.unstack(yerr)
         columns = agg.columns
         panel = {}
 
